
sq=1 ; export sq

for p1 in 0 1 2 3 4 5
do
   for p2 in 0 1 2 3 4 5 6 7 8 9
   do
   sq=`expr $p1 \* 10 \+ $p2` 

   dc -e "40 k $sq v n"
   echo "  | $sq^(1/2)"
   done
done


for p1 in 0 1 2 3 4 5
do
   for p2 in 0 1 2 3 4 5 6 7 8 9
   do
   sq=`expr $p1 \* 10 \+ $p2` 

   dc -e "40 k 1 $sq / v n"
   echo "  | $sq^(-1/2)"
   done
done

