                          /* usage.h,  6 Apr 10 */

extern BOOL do_usage_value,
	    do_usage_file;
extern int total_dir_seen;

extern void cnt_val_usage(DIGIT_CHAR * mantissa,
		      int len_mantissa,
		      DIGIT_CHAR * exponent);
extern void cnt_file_processed(char *filename);
extern void cnt_file_not_processed(char *filename);
extern void print_usage(void);

