#
# signifmerge.awk, 17 Apr 10

$1 == ".File:" {
	if ($2 != "signif_digits")
	   {
	   in_signif_digits=0
	   next
	   }
	in_signif_digits=1
	num_files++
	next
	}

in_signif_digits == 1 && ($1 != "Characters") {
	char_percent[$1]+=$3
	expr_percent[$1]+=$5
	init_percent[$1]+=$7
	total_literals+=$2
	next
	}

	{
	next
	}

END {
#	print total_literals
	for (len=1; len < 20; len++)
	   printf("%3d  %5.2f  %5.2f  %5.2f\n",  \
		len, char_percent[len]/num_files, expr_percent[len]/num_files, init_percent[len]/num_files)
	}

