                      /* fltval.h, 12 May 10 */


extern const char *opt_fuzz_str;
extern fp_t default_fuzz_val;

extern void fp_init(fp_t * numeric_val);
extern void fp_print(fp_t numeric_val);
extern double fp_to_double(fp_t numeric_val);
extern void fp_to_string(DIGIT_CHAR *val_str,
			 fp_t numeric_val,
			 int signif_digits);
extern BOOL string_to_fp(fp_t * numeric_val,
			 const DIGIT_CHAR *mantissa,
			 int mant_len,
			 const DIGIT_CHAR *exponent);
extern void set_fuzz_val(fp_t *fuzz_val, const char *fuzz_str);
extern int fp_almost_eq(fp_t operand_1, fp_t operand_2, fp_t fuzz_percent);
extern int fp_eq(fp_t left_operand, fp_t right_operand);
extern void fp_assign(fp_t * lhs, fp_t rhs);
extern void init_fuzz_percent(const char *duzz_str);

