                         /* matchnum.h, 29 May 11 */

extern BOOL found_match;
extern BOOL do_print_match;
extern BOOL do_print_value;
extern BOOL do_print_word;
extern BOOL match_mantissa;
extern BOOL match_value;
extern int min_levkey_dist;
extern int num_simp_digits;
extern DIGIT_CHAR simp_digit_seq[MAX_NUM_DIGITS];

extern void init_match(void);
extern void simplify_match_num(DIGIT_CHAR *some_num, int len_some_num);
extern void match_number(DIGIT_CHAR * mantissa,
			 int len_mantissa,
			 DIGIT_CHAR * exponent);
extern BOOL save_match(enum match_enum match_kind,
		       struct num_db_rec *spec_num_ptr,
		       fp_t seen_val,
		       DIGIT_CHAR *mantissa,
		       int len_mantissa,
		       DIGIT_CHAR *exponent);

