                      /* numbers.c,  7 May 10 */

#include "global.h"
#include "printnum.h"
#include "options.h"
#include "usage.h"
#include "singlefile.h"
#include "chkfiles.h"


int main(int argc, char *argv[])
{
init(argc, argv);
process_arg_files();
if (!do_mk_ndb)
   {
   print_summary();
   print_usage();
   }

return 0;
}

