
pow=1 ; export pow

   for p1 in 2 3 4 5 6 7 8 9 10 11 12 13 14
   do
   pow=`expr $p1 \* $pow` 

   dc -e "40 k 1 $pow / p"
   echo "  | 1/$p1!"
   done

exit

for p1 in 0 1 2 3 4 5 6
do
   for p2 in 0 1 2 3 4 5 6 7 8 9
   do
   pow=`expr $p1 \* 10 \+ $p2` 

   dc -e "70 k 2 1 $pow / p ^ p"
   echo "  | 2^(1/$pow)"
   done
done

for p1 in 0 1 2 3 4 5 6
do
   for p2 in 0 1 2 3 4 5 6 7 8 9
   do
   pow=`expr $p1 \* 10 \+ $p2` 

   dc -e "70 k 2 $pow ^ p"
   echo "  | 2^-$pow"
   done
done

