#
# need.sh, 18 May 11

echo "need"
numbers -ndb need.ndb need_1.c

echo "-need perc 40"
numbers -ndb need.ndb need_1.c -need perc 40

echo "-need perc 50"
numbers -ndb need.ndb need_1.c -need perc 50

echo "-need perc 60"
numbers -ndb need.ndb need_1.c -need perc 60

